@@ -1,4 +1,4 @@
-package Pcore::PDF v0.8.0;
+package Pcore::PDF v0.8.1;
 
 use Pcore -dist, -class, -const, -res;
 use Config;
@@ -42,8 +42,7 @@ has bin         => 'princexml';
 has max_threads => P->sys->cpus_num;
 has page_size   => $PAGE_SIZE->{A4};
 
-has _threads   => 0;
-has _signal    => sub { Coro::Signal->new };
+has _semaphore => sub ($self) { Coro::Semaphore->new( $self->{max_threads} ) }, is => 'lazy';
 has _princexml => ();
 
 # NOTE job config example
@@ -96,9 +95,9 @@ has _princexml => ();
 # };
 
 sub generate_pdf ( $self, $html ) {
-    $self->{_signal}->wait if $self->{max_threads} && $self->{_threads} == $self->{max_threads};
 
-    $self->{_threads}++;
+    # block thread
+    my $guard = $self->{max_threads} && $self->_semaphore->guard;
 
     my $princexml;
 
@@ -112,10 +111,6 @@ sub generate_pdf ( $self, $html ) {
 
     push $self->{_princexml}->@*, $princexml;
 
-    $self->{_threads}--;
-
-    $self->{_signal}->send;
-
     return $res;
 }
 
